(* Dans ce module il y a des trucs finalement inutilisés, et des trucs
 * qui ne servent qu'en interne.
 *
 * Voir le mli pour ce qui est vraiment utile, et pour des commentaires. *)

module type T = sig val n : int end

module Make (M:T) = struct

  open M

  type pos = int*int

  type path = pos list

  let around x =
    let xs = [x] in
    let xs = if x>1 then x-1 :: xs else xs in
    let xs = if x<n then x+1 :: xs else xs in
      xs

  let rec range i j =
    if i>j then [] else i :: range (i+1) j

  let rec iter_range i j f =
    if i<=j then begin
      f i ;
      iter_range (i+1) j f
    end

  let rec product l1 = function
    | [] -> []
    | x::l2 ->
        List.map (fun y -> y,x) l1 @
        product l1 l2

  let all_pos =
    product (range 1 n) (range 1 n)

  let iter_all_pos f =
    iter_range 1 n (fun i -> iter_range 1 n (fun j -> f (i,j)))

  let neighbours (x,y) =
    let xs = around x in
    let ys = around y in
    let pos = product xs ys in
    let pos = List.filter (fun p -> p <> (x,y)) pos in
      pos

  (** Hamiltonian path representation *)

  module PSet = Set.Make(struct type t = pos let compare = compare end)

  type hpath = PSet.t * path

  let path_of_hpath = snd

  let empty_hpath = PSet.empty,[]

  let hpath_step (visited,path) pos =
    if PSet.mem pos visited then None else
      Some (PSet.add pos visited, pos::path)

  let hpath_next (visited,path) =
    let next =
      match path with
        | [] -> all_pos
        | pos::_ ->
            let next = neighbours pos in
              List.filter (fun p -> not (PSet.mem p visited)) next
    in
      List.map
        (fun n ->
           let visited = PSet.add n visited in
           let path = n::path in
             (visited,path))
        next

  (** Enumerate all Hamiltonian paths *)

  let rec paths f = function
    | [] -> ()
    | hpath::todo ->
        f (snd hpath) ;
        let next = hpath_next hpath in
        paths f (List.rev_append next todo)

  let paths pos f = paths f [(PSet.singleton pos,[pos])]

  (** Pretty printing *)

  let pp_pos chan (x,y) = Format.fprintf chan "%d%d" x y

  let rec pp_path chan = function
    | [] -> ()
    | [p] -> pp_pos chan p
    | p::l -> Format.fprintf chan "%a,%a" pp_path l pp_pos p

end
