(* Représentation d'une grille (sans contenu aux cases)
 * et de chemins dans la grille.
 * On utilise un foncteur pour ne pas spécifier n sans arrêt. *)

module type T = sig val n : int end
module Make : functor (M : T) -> sig

  type pos = int * int
  val all_pos : pos list
  val neighbours : pos -> pos list

  (** Arbitrary path = list of contiguous positions *)
  type path = pos list

  (** Hamiltonian path, never visiting the same position twice *)
  type hpath

  val path_of_hpath : hpath -> path
  val empty_hpath : hpath

  (** Try to extend an hpath to a next position *)
  val hpath_step : hpath -> pos -> hpath option

  (** Get all possible extensions *)
  val hpath_next : hpath -> hpath list

  (** Pretty printers *)

  val pp_pos : Format.formatter -> pos -> unit
  val pp_path : Format.formatter -> pos list -> unit

end
