
let () =
  let w = "foo" in
  let t = Trie.singleton w in
    assert (Trie.mem w t) ;
    assert (t = Trie.add w t) ;
    let t = Trie.add "bar" t in
      assert (Trie.mem w t && Trie.mem "bar" t) ;
      let t = Trie.add "barre" t in
      assert (Trie.mem "barre" t && Trie.mem "bar" t) ;
      assert (t = Trie.add "bar" t) ;
      assert (t = Trie.add "barre" t) ;
      Printf.printf "Trie tests OK\n"

let () =
  assert (Utils.group [(1,"a");(1,"b");(2,"ab");(3,"abc");
                       (4,"abcd");(4,"abcd")] =
          [ 1, ["b";"a"] ;
            2, ["ab"] ;
            3, ["abc"] ;
            4, ["abcd";"abcd"] ]) ;
  assert (Utils.group [(1,"a");(1,"b");(2,"ab");(3,"abc");
                       (4,"abcd");(4,"abcd");(5,"abcde")] =
          [ 1, ["b";"a"] ;
            2, ["ab"] ;
            3, ["abc"] ;
            4, ["abcd";"abcd"] ;
            5, ["abcde"] ])

let () =
  assert (Utils.filter_some [Some 1; None; Some 2] = [2;1])

let () =
  if Sys.file_exists "words.trie" then
    let t = Trie.load "words.trie" in
      Printf.printf
        "Words trie has %d entries.\n"
        (Trie.size t)

let () =
  Random.self_init () ;
  let test c =
    let count = ref 0 in
    let n = 1000000 in
    let margin = 0.1 /. float Scrabble.total in
      for i = 1 to n do
        if c = Scrabble.random_char () then incr count
      done ;
      let theory =
        float (List.assoc c Scrabble.distribution) /. float Scrabble.total
      in
      let practice = float !count /. float n in
        assert (abs_float (theory -. practice) < margin)
  in
    test 'E' ; test 'M' ; test 'X' ; test 'D' ;
    Printf.printf "Scrabble generator OK\n"
