(** Implémentation des tries
  * Une trie est un ensemble de mots organisé comme une arbre
  * ce qui permet de faire une recherche par préfixe. *)

type t

val size : t -> int

(** Creation de tries *)

val empty : t
val singleton : ?start:int -> string -> t

(** Test d'appartenance et ajout *)

val mem : string -> t -> bool
val add : string -> t -> t

(** Liste des premières lettres possibles *)

val next_letters : t -> char list

(** Passage de S à un sous-ensemble a^-1 S,
  * avec type option ou renvoi d'exception Not_found dans le cas vide. *)

val step : char -> t -> t
val step_option : char -> t -> t option

(** (Dé)Sérialisation *)

val save : t -> filename:string -> unit
val load : filename:string -> t
