(** Various utilities *)

(** Simple list utilities *)

let rec sorted_insert ~compare elt = function
  | [] -> [elt]
  | hd::tl ->
      match compare hd elt with
        | 1 -> hd :: sorted_insert ~compare elt tl
        | _ -> elt :: hd :: tl

let rec truncate size l =
  if size <= 0 then [] else
    match l with
      | [] -> []
      | hd :: tl -> hd :: truncate (size-1) tl

let group l =
  let rec aux h l = function
    | (h',x')::tl ->
        assert (h' >= h) ;
        if h=h' then
          aux h (x'::l) tl
        else
          (h,l) :: aux h' [x'] tl
    | [] -> [h,l]
  in
    match l with
      | [] -> []
      | (h,x)::tl -> aux h [x] tl

let filter_some l =
  List.fold_left
    (fun l -> function
       | None -> l
       | Some x -> x::l)
    []
    l

(** [append_some l tail = List.rev_append (List.filter ((<>) None) l) tail] *)
let append_some l tail =
  List.fold_left
    (fun tail -> function
       | Some i -> i::tail
       | None -> tail)
    tail l

(** Primitives de choix en success/failure *)

(** Backtracking map in success/failure style. *)
let rec map f l succ fail =
  match l with
    | [] -> succ [] fail
    | hd::tl ->
        f hd
          (fun hd' k -> map f tl (fun tl' k -> succ (hd'::tl') k) k)
          fail

(** Return a char in 'a'..'z'. *)
let random_char succ fail =
  let a = int_of_char 'a' in
  let rec aux k =
    if k = 26 then fail () else
      succ (char_of_int (a+k)) (fun () -> aux (k+1))
  in
    aux 0

(** Pick an element in a list. *)
let rec pick l ~succ ~fail =
  match l with
    | [] -> fail ()
    | hd::tl -> succ hd (fun () -> pick tl ~succ ~fail)

(** Utilitaires pour assignements *)

(** Convert assignment hash to list. *)
let assignment_to_list h =
  Hashtbl.fold (fun k v l -> (k,v)::l) h []

(** Pretty print assignment list. *)
let rec pp_assignment chan = function
  | [] -> ()
  | [k,v] -> Format.fprintf chan "%c=%c" k v
  | (k,v)::tl -> Format.fprintf chan "%c=%c,%a" k v pp_assignment tl
